﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

namespace i388
{
    /// <summary>
    /// Egy adott tájolású térképet és pontjait reprezentáló objektum.
    /// </summary>
    public class Térkép
    {
        /// <summary>
        /// Megadja a térkép pontjait.
        /// </summary>
        public Dictionary<int, Vektor2> Pontok { get; private set; }

        /// <summary>
        /// Megadja, hogy az adott pontról tartalmaz -e információt a térkép.
        /// </summary>
        public BitArray PontInformáció { get; private set; }

        /// <summary>
        /// Megadja a térkép tájolási irányának az egységvektorát.
        /// </summary>
        public Vektor2 Tájolás { get; set; }

        public Térkép(int[] adatok, List<int> összesPont)
        {
            int darabszám = adatok.First();

            this.Pontok = new Dictionary<int, Vektor2>();

            List<int> pontSorszámok = new List<int>();

            for (int i = 0; i < darabszám; i++)
            {
                pontSorszámok.Add(adatok[3 * i + 1]);
                this.Pontok.Add(adatok[3 * i + 1], new Vektor2(adatok[3 * i + 2], adatok[3 * i + 3]));
            }

            this.PontInformáció = new BitArray(összesPont.Select(pont => pontSorszámok.Contains(pont)).ToArray());
        }

        /// <summary>
        /// Hozzáad egy pontot a térképhez.
        /// </summary>
        /// <param name="index">A pont indexe.</param>
        /// <param name="pont">A pont koordinátái.</param>
        /// <param name="összesPont">A pontok halmaza, amelyben a pont indexelve van.</param>
        public void PontHozzáadás(int index, Vektor2 pont, List<int> összesPont)
        {
            // pont hozzáadása a pontokhoz
            this.Pontok.Add(index, pont);

            // pont hozzáadása a pont információs vektorhoz
            this.PontInformáció.Set(összesPont.IndexOf(index), true);
        }

        /// <summary>
        /// Megadja azon pontok halmazát, amelyet mindkét térkép tartalmaz.
        /// </summary>
        /// <param name="térkép">A vizsgálandó térkép.</param>
        public int[] Kapcsolat(Térkép térkép)
        {
            BitArray kapcsolat = new BitArray(this.PontInformáció).And(térkép.PontInformáció);
            IEnumerable<Tuple<bool, int>> lista = kapcsolat
                .Cast<bool>()
                .Select((bit, index) => new Tuple<bool, int>(bit, index));
            return lista
                .Where(pár => pár.Item1)
                .Select(pár => pár.Item2 + 1)
                .ToArray();
        }
    }

    class Program
    {
        /// <summary>
        /// A tájolás karakterét vektorrá alakítja.
        /// </summary>
        /// <param name="karakter">A tájolást jellemző karakter.</param>
        /// <returns>A tájolás irányába mutató vektor.</returns>
        static Vektor2 Tájolás(char karakter)
        {
            switch (karakter)
            {
                case 'E':
                    return Vektor2.Észak;
                case 'D':
                    return Vektor2.Dél;
                case 'K':
                    return Vektor2.Kelet;
                case 'N':
                    return Vektor2.Nyugat;
                default:
                    return Vektor2.NullVektor;
            }
        }

        /// <summary>
        /// A tájolás vektorát karakteres formába alakítja.
        /// </summary>
        /// <param name="vektor">A tájolás irányába mutató vektor.</param>
        /// <returns>A tájolást jellemző karakter.</returns>
        static char Tájolás(Vektor2 vektor)
        {
            if (vektor == Vektor2.Észak) return 'E';
            else if (vektor == Vektor2.Dél) return 'D';
            else if (vektor == Vektor2.Kelet) return 'K';
            else if (vektor == Vektor2.Nyugat) return 'N';
            return 'X';
        }

        static void Main(string[] args)
        {
            List<Térkép> térképek = new List<Térkép>();

            List<int> összesPont = new List<int>();

            // pontok hozzáadása
            using (System.IO.StreamReader olvasó = new System.IO.StreamReader("terkep.be"))
            {
                string[] fejléc = olvasó.ReadLine().Split(' ');

                int darabszám = int.Parse(fejléc[0]);

                for (int i = 0; i < darabszám; i++)
                {
                    összesPont.AddRange(olvasó.ReadLine().Split(' ')
                        .Select(szó => int.Parse(szó))
                        .Where((szám, sorszám) => sorszám % 3 == 1));
                }

                összesPont = összesPont.Distinct().OrderBy(szám => szám).ToList();
            }

            // térképek hozzáadása
            using (System.IO.StreamReader olvasó = new System.IO.StreamReader("terkep.be"))
            {
                string[] fejléc = olvasó.ReadLine().Split(' ');

                int darabszám = int.Parse(fejléc[0]);
                char tájolás = fejléc[1].First();

                for (int i = 0; i < darabszám; i++)
                {
                    térképek.Add(new Térkép(olvasó.ReadLine().Split(' ').Select(szó => int.Parse(szó)).ToArray(), összesPont));
                }

                térképek.First().Tájolás = Tájolás(tájolás);
            }

            // akkor igaz, ha találtunk kapcsolódó térképet
            bool talált = false;

            do
            {
                // alaphelyzetben nem találtunk térképet
                talált = false;

                // az első térkép rögzített, a másodikat a második elemtől indulva választjuk
                for (int i = 1; i < térképek.Count; i++)
                {
                    // ha már tudjuk a tájolását, nem kell a térképre helyezni a pontokat
                    if (térképek[i].Tájolás != Vektor2.NullVektor) continue;

                    // a két térkép közös pontjai
                    int[] pontok = térképek.First().Kapcsolat(térképek[i]);

                    // ha van két közös pont, meg tudjuk mondani a
                    if (pontok.Length >= 2)
                    {
                        // a két térképen a két pont irányvektora
                        Vektor2 térkép1Irány = térképek.First().Pontok[pontok[0]] - térképek.First().Pontok[pontok[1]];
                        Vektor2 térkép2Irány = térképek[i].Pontok[pontok[0]] - térképek[i].Pontok[pontok[1]];

                        // a forgatás transzformáció a két térkép között
                        Forgatás forgatás = new Forgatás(térkép2Irány, térkép1Irány);

                        // a tájolást forgatással megkapjuk
                        térképek[i].Tájolás = térképek.First().Tájolás.Forgatás(forgatás);

                        // az első és a második térkép azonos indexű pontja
                        Vektor2 térkép1Pont = térképek.First().Pontok[pontok[0]];
                        Vektor2 térkép2Pont = térképek[i].Pontok[pontok[0]];

                        // a második térkép szerint a pont elforgatva
                        Vektor2 térkép2PontForgatva = térkép2Pont.Forgatás(forgatás);

                        // a két térkép közötti eltolás transzformáció
                        Vektor2 eltolás = térkép1Pont - térkép2PontForgatva;

                        // ráhelyezzük a kiválasztott térkép pontjait
                        foreach (KeyValuePair<int, Vektor2> pont in térképek[i].Pontok)
                        {
                            // azokat a pontokat helyezzük rá a térképre, amelyek eddig nem szerepeltek
                            if (térképek.First().Pontok.ContainsKey(pont.Key)) continue;

                            // a pont hozzáadása a térképhez
                            térképek.First().PontHozzáadás(pont.Key, pont.Value.Forgatás(forgatás) + eltolás, összesPont);
                        }

                        // találtunk térképet
                        talált = true;
                    }
                }
            } while (talált); // addig járjuk körbe, ameddig találunk térképet

            // az eredmények fájlba írása
            using (System.IO.StreamWriter író = new System.IO.StreamWriter("terkep.ki"))
            {
                // a térképeken végigiterálunk
                foreach (Térkép térkép in térképek)
                {
                    // a tájolást fájlba írjuk
                    író.WriteLine(Tájolás(térkép.Tájolás));
                }
            }
        }
    }
}
