﻿using System;

namespace i388
{
    /// <summary>
    /// Reprezentál egy kétdimenziós vektort.
    /// </summary>
    public struct Vektor2
    {
        /// <summary>
        /// A vektor X-irányú komponense.
        /// </summary>
        public float X;

        /// <summary>
        /// A vektor Y-irányú komponense.
        /// </summary>
        public float Y;

        /// <summary>
        /// Megadja az északi égtáj irányába mutató egységvektort.
        /// </summary>
        public static readonly Vektor2 Észak = new Vektor2(0, 1);

        /// <summary>
        /// Megadja az déli égtáj irányába mutató egységvektort.
        /// </summary>
        public static readonly Vektor2 Dél = new Vektor2(0, -1);

        /// <summary>
        /// Megadja az keleti égtáj irányába mutató egységvektort.
        /// </summary>
        public static readonly Vektor2 Kelet = new Vektor2(1, 0);

        /// <summary>
        /// Megadja az nyugati égtáj irányába mutató egységvektort.
        /// </summary>
        public static readonly Vektor2 Nyugat = new Vektor2(-1, 0);

        /// <summary>
        /// Megadja a nulla hosszúságú vektort.
        /// </summary>
        public static readonly Vektor2 NullVektor = new Vektor2(0, 0);

        /// <summary>
        /// Megadja a vektor origótól számított hosszát.
        /// </summary>
        public float Hossz
        {
            get
            {
                return (float)Math.Sqrt(this * this);
            }
        }
            
        /// <summary>
        /// Megadja a vektor irányába mutató egység hosszú vektort.
        /// </summary>
        public Vektor2 IrányVektor
        {
            get
            {
                return this / this.Hossz;
            }
        }

        /// <summary>
        /// Létrehoz egy új kétdimenziós vektort a komponensei által.
        /// </summary>
        /// <param name="x">A vektor X-irányú komponense.</param>
        /// <param name="y">A vektor Y-irányú komponense.</param>
        public Vektor2(float x, float y)
        {
            this.X = x;
            this.Y = y;
        }

        /// <summary>
        /// Létrehoz egy új kétdimenziós vektort a komponensei által.
        /// </summary>
        /// <param name="x">A vektor X-irányú komponense.</param>
        /// <param name="y">A vektor Y-irányú komponense.</param>
        public Vektor2(int x, int y)
        {
            this.X = x;
            this.Y = y;
        }

        public static Vektor2 operator +(Vektor2 bal, Vektor2 jobb)
        {
            return new Vektor2(bal.X + jobb.X, bal.Y + jobb.Y);
        }

        public static Vektor2 operator -(Vektor2 bal, Vektor2 jobb)
        {
            return new Vektor2(bal.X - jobb.X, bal.Y - jobb.Y);
        }

        public static Vektor2 operator /(Vektor2 bal, float jobb)
        {
            return new Vektor2(bal.X / jobb, bal.Y / jobb);
        }

        public static bool operator ==(Vektor2 bal, Vektor2 jobb)
        {
            return bal.X == jobb.X && bal.Y == jobb.Y;
        }

        public static bool operator !=(Vektor2 bal, Vektor2 jobb)
        {
            return bal.X != jobb.X || bal.Y != jobb.Y;
        }

        public static float operator *(Vektor2 bal, Vektor2 jobb)
        {
            return bal.X * jobb.X + bal.Y * jobb.Y;
        }

        /// <summary>
        /// Elforgatja a vektort a forgatásnak megfelelő módon az egyik égtáj irányába.
        /// </summary>
        /// <param name="forgatás">A forgatás módja.</param>
        /// <returns>Az elforgatott vektor.</returns>
        public Vektor2 Forgatás(Forgatás forgatás)
        {
            Vektor2 vektor = new Vektor2(this.X, this.Y);

            for (int i = 0; i < forgatás.Szám; i++)
            {
                vektor = new Vektor2(-vektor.Y, vektor.X);
            }

            return vektor;
        }

        public override string ToString()
        {
            return string.Format("({0}; {1})", this.X, this.Y);
        }
    }
}